//---------------------------------------------------------------------------
// Copyright (C) 2000 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  LD2433.c - This utility uses TMEX to load the memory of a DS2433 from
//             a file.
//
//  Compiler:  Microsoft Visual C 
//  Version:   1.01
//  History: 1.00 -> 1.01 Add binary file support
//

#define TMEXUTIL

#include "ibtmexcw.h"   

// constants 
#define MAX_WRITEBUF           1024
#define MAX_FILEBUF            5000
#define DS2433_FAMILY          0x23

// Local Function Prototypes 
int FindFirstFamily(short, long);
void Exit_Prog(char *, short);
short ToHex(char);
int iButtonFound(short);
int ParseData(char *, int, uchar *, int);
ushort crc16(int data);
int WritePages(int, uchar *);
int WriteScratch(long, int, uchar *);
int CopyScratch(long, int, uchar *);
int ReadMemory(int, uchar *);

// Global Variables to hold list information 
short PortType, PortNum;
uchar state_buffer[5120];
ushort CRC16;

//----------------------------------------------------------------------
//  This is the Main routine for LD2433
//
void main(short argc, char **argv)
{
   short writelen=0,i,fllen,loopcnt=0,newlen,binary=0;
   char tstr[80];       
   char filebuf[MAX_FILEBUF];
   uchar writebuf[MAX_WRITEBUF];
   uchar verifybuf[MAX_WRITEBUF];
   FILE *fp;

   // check arguments to see if request instruction with '?' or too many 
   if ((argc < 2) || ((argc == 2) && (argv[1][0] == '?' || argv[1][1] == '?')))
   {
       printf("\nusage: LD2433 InputFile <BINARY>\n"
              "  - required argument 1 specifies the InputFile to write to the device\n"
              "  - optional argument 2 is a flag to indicate InputFile is a binary file\n"
              "  - version 1.01\n");
       Exit_Prog("",BAD_ARGUMENT);
   }

   // read the default PortNum and PortType 
   if (!TMReadDefaultPort(&PortNum,&PortType))
       Exit_Prog("ERROR, Could not read the default PortNum and PortType from registry!\n",
               NO_DRIVERS);

   // print a header   
   printf("\nLD2433 Version 1.01\n"
         "Dallas Semiconductor Corporation\n"
         "Copyright (C) 2000\n\n");
   printf("Default port number: %d    port type: %d\n",PortNum,PortType);
   Get_Version(tstr);
   printf("TMEX main driver: %s\n",tstr);
   printf("TMEX adapter driver, Type%d:",PortType);
   if (TMGetTypeVersion(PortType,tstr) < 0)
      Exit_Prog("No Hardware Driver for this type found!",NO_DRIVERS);
   printf(" %s\n\n",tstr);

   // check for binary flag
   if (argc > 2)
   {
      binary = TRUE;
      printf("BINARY input file specified.\n");
   }
   
   // open the input file and read it 
   fp=fopen(argv[1],"rb");
   if(fp == NULL)
      Exit_Prog("Could not open specified InputFile!\n",BAD_ARGUMENT);
   else
      printf("File '%s' opened to read.\n", argv[1]);

   // read the file 
   fllen = fread(filebuf,1,MAX_FILEBUF,fp);
   // close the file 
   fclose(fp);
   printf("File '%s' closed.\n",argv[1]);

   // parse if not binary
   if (!binary)
   {
      // attempt to convert the text into an array of uchars 
      writelen = ParseData(filebuf, fllen, writebuf, MAX_WRITEBUF);

      // check to verify the file data is correct  
      if (writelen == 0)
         Exit_Prog("InputFile not in the correct format!\n",BAD_ARGUMENT);
   }
   else
   {
      writelen = fllen;
      for (i = 0; i < writelen; i++)
         writebuf[i] = filebuf[i];
   }

   // check if too long
   if (writelen > 512)
   {
      printf("InputFile has TOO much data, truncating to 512 bytes.\n");
      writelen = 512;
   }

   // pad the data out the the end of a page
   if (writelen % 32)
   {
      newlen = (((writelen / 32) + 1) * 32);
      for (i = writelen; i < newlen; i++)
         writebuf[i] = 0;
      printf("\nData padded with %d bytes (0x00's) to come to end of page.\n", newlen - writelen);
      writelen = newlen;
   }

   // print out the buffer to write  
   printf("\n%d bytes of data to write starting at address 0:\n",writelen);   
   for (i = 0; i < writelen; i++)
   {
      printf("%02X",writebuf[i]);
      if (((i + 1) % 16) == 0)
         printf("\n");
      else if (((i + 1) % 8) == 0)
         printf("-");
      else
         printf(" ");
   }
   printf("\n\n");

   // find a device 
   if (iButtonFound(DS2433_FAMILY))                          
   {   
      // write and then read data 
      do 
      {
         // write
         printf("Writing data...\n");
         if (WritePages(writelen / 32, writebuf))
         {
            printf("Write successful\nRead back data...\n");

            // read back to verify (dedundent)
            if (ReadMemory(writelen / 32, verifybuf))
            {
               printf("Read back complete\n");
               // loop to compare
               for (i = 0; i < writelen; i++)
                  if (writebuf[i] != verifybuf[i])
                     break;

               if (i != writelen)
                  printf("ERROR, data read back incorrect, retry %d\n",loopcnt + 1);
               else
                  Exit_Prog("Data read back CORRECT!\n\nEnd program normally\n\n", NORMAL_EXIT);
            }
            else
               printf("ERROR reading memory, retry %d\n",loopcnt + 1);
         }
         else
            printf("Error during write, retry %d\n",loopcnt + 1);
      }
      while (loopcnt++ < 10);
   }   
   else
      Exit_Prog("\n",-NO_DEVICE);

   Exit_Prog("Exceeded the number of retries, failed to write device!\n",-FILE_WRITE_ERR);
}

//----------------------------------------------------------------------
// Check if a device of the correct family has been found
//    
// Returns: TRUE (1) success
//          FALSE (0) failed
//                                                       
int iButtonFound(short family)
{
   int printmsg=1, found;
   long session_handle;                         

   // loop to get a session handle
   found = FALSE;
   for (;;)
   {
      // attempt to get a session 
      session_handle = TMExtendedStartSession(PortNum,PortType,NULL);
      if (session_handle > 0)  
      {
         // setup the MicroLan 
         if (TMSetup(session_handle) == 1)
         {
            printf("Searching for device type %02X...\n\n",family);
            
            // look for the first iButton 
            if (FindFirstFamily(family,session_handle))
            {
               found = TRUE;
               break;
            }
            // iButton not on MicroLan 
            else   
            {
               printf("Device not found on 1-Wire Net!"); 
               break;
            }
         }   
         // MicroLan port not valid 
         else   
         {
            printf("1-Wire Net setup failed!"); 
            break;
         }
      }
      else if (session_handle == 0)
      {                            
         // check if need to print the waiting message 
         if (printmsg)
         {  
            printmsg = FALSE;
            printf("\nWaiting to get access to the 1-Wire Net\n\n");
         }
      }
      else if (session_handle < 0)   
      {
         MessageBeep(MB_ICONEXCLAMATION);
         printf("No Hardware Driver for this type found!");
         break;
      }
   }

   // end the session if one is open 
   TMEndSession(session_handle);

   return found;
}

//----------------------------------------------------------------------
// Parse the raw file data in to an array of uchar's.  The data must
// be hex characters possibly seperated by white space
//     12 34 456789ABCD EF
//     FE DC BA 98 7654 32 10
// would be converted to an array of 16 uchars.
// Returns the array length.  If an invalid hex pair is found then the
// return will be 0.
//
int ParseData(char *inbuf, int insize, uchar *outbuf, int maxsize)
{
   int ps, outlen=0, gotmnib=0;
   uchar mnib;

   // loop until end of data
   for (ps = 0; ps < insize; ps++)
   {
      // check for white space 
      if (isspace(inbuf[ps]))
         continue;
      // not white space, make sure hex
      else if (isxdigit(inbuf[ps]))
      {
         // check if have first nibble yet
         if (gotmnib)
         {  
            // this is the second nibble
            outbuf[outlen++] = (mnib << 4) | ToHex(inbuf[ps]);
            gotmnib = 0;
         }
         else
         {
            // this is the first nibble
            mnib = (uchar)ToHex(inbuf[ps]);
            gotmnib = 1;
         }
      }
      else
         return 0;

      // if getting to the max return what we have
      if ((outlen + 1) > maxsize)
         return outlen;
   }

   return outlen;
}

//----------------------------------------------------------------------
// Find the first device with a particular family
//
// Returns: TRUE (1) success
//          FALSE (0) failed
//
int FindFirstFamily(short family, long session_handle)
{
   int j;
   short ROM[9];
   
   // set up to find the first device with family 'family 
   if (TMFamilySearchSetup(session_handle,&state_buffer,family) == 1)
   { 
      // get first device in list with the specified family 
      if (TMNext(session_handle,state_buffer) == 1)
      {  
         // read the rom number 
         ROM[0] = 0;
         TMRom(session_handle,state_buffer,ROM);   
         
         // check if correct type 
         if ((family & 0x7F) == (ROM[0] & 0x7F))
         {
            printf("Device 1-Wire Net address: ");  
            for (j = 7; j >= 0; j--)
               printf("%02X",ROM[j]);
            printf("\n\n");
            return 1;
         }        
      }
   }
     
   // failed to find device of that type 
   return 0;                              
}


//--------------------------------------------------------------------------
//  Prints a message, beeps and exits the program.
//
void Exit_Prog(char *msg, short errcd)
{
   // print the message left justified 
   printf("\r%s",msg);
   
   //  error beep if return code is not 0 
   if (errcd != NORMAL_EXIT)
     MessageBeep(MB_ICONEXCLAMATION);
   
   //  return error code passed 
   exit(errcd);
}

//------------------------------------------------------------------------
// Convert 1 hex character to binary.
//
short ToHex(char ch)
{
   if ((ch >= '0') && (ch <= '9'))
      return ch - 0x30;
   else if ((ch >= 'A') && (ch <= 'F'))
      return ch - 0x37;
   else if ((ch >= 'a') && (ch <= 'f'))
      return ch - 0x57;
   else
      return 0;
}

//--------------------------------------------------------------------------
// Write specified number of pages
//    
// Returns: TRUE (1) success
//          FALSE (0) failed
//
int WritePages(int num_pgs, uchar *buf)
{
   int pg,rt=TRUE;
   short printmsg=1;
   long session_handle;                         

   // loop to get a session handle
   for (;;)
   {
      // attempt to get a session 
      session_handle = TMExtendedStartSession(PortNum,PortType,NULL);
      if (session_handle > 0)  
      {
         // write to each page
         for (pg = 0; pg < num_pgs; pg++)
         {
            if (!WriteScratch(session_handle, pg, &buf[pg * 32]))
            {
               rt = FALSE;
               break;
            }

            if (!CopyScratch(session_handle, pg, &buf[pg * 32]))
            {
               rt = FALSE;
               break;
            }
         }
         break;
      }
      else if (session_handle == 0)
      {                            
         // check if need to print the waiting message 
         if (printmsg)
         {  
            printmsg = FALSE;
            printf("\nWaiting to get access to the 1-Wire Net\n\n");
         }
      }
      else if (session_handle < 0)   
      {
         MessageBeep(MB_ICONEXCLAMATION);
         printf("No Hardware Driver for this type found!");
         break;
      }
   }

   // end the session if one is open 
   TMEndSession(session_handle);

   return rt;
}

//--------------------------------------------------------------------------
// Write scratch
//    
// Returns: TRUE (1) success
//          FALSE (0) failed
//
int WriteScratch(long session_handle, int pg, uchar *buf)
{
   int i,cnt=0;
   uchar pbuf[50];

   // construct a packet to send  
   if (TMAccess(session_handle,&state_buffer) == 1)
   {
      // construct a packet to send  
      pbuf[cnt++] = 0x0F; // write scratch command 
      pbuf[cnt++] = (pg << 5) & 0xFF; // address 1 
      pbuf[cnt++] = ((pg >> 3 ) & 0xFF); // address 2 

      // data to write  
      for (i = 0; i < 32; i++)
        pbuf[cnt++] = (uchar)(buf[i]);  

      // run the CRC over this part
      CRC16 = 0;
      for (i = 0; i < cnt; i++)
         crc16(pbuf[i]);

      // crc bytes
      pbuf[cnt++] = 0xFF;
      pbuf[cnt++] = 0xFF;
      
      // perform the block 
      if (TMBlockStream(session_handle,pbuf,(short)(cnt)) != (short)(cnt))
         return FALSE;

      // check CRC 
      for (i = cnt - 2; i < cnt; i++)
         crc16(pbuf[i]);

      // return result of CRC
      return (CRC16 == 0xB001);
   }
   else
      return FALSE;
}

//--------------------------------------------------------------------------
// Copy scratch
//    
// Returns: TRUE (1) success
//          FALSE (0) failed
//
int CopyScratch(long session_handle, int pg, uchar *buf)
{
   int cnt=0;
   uchar pbuf[50], ch;

   // construct a packet to send  
   if (TMAccess(session_handle,&state_buffer) == 1)
   {
      // construct a packet to send  
      pbuf[cnt++] = 0x55; // write scratch command 
      pbuf[cnt++] = (pg << 5) & 0xFF; // address 1 
      pbuf[cnt++] = ((pg >> 3 ) & 0xFF); // address 2 
      pbuf[cnt++] = 0x1F;  // offset

      // perform the block (don't send last byte)
      if (TMBlockStream(session_handle,pbuf,(short)(cnt-1)) != 
                        (short)(cnt-1))
         return FALSE;

      // prepare the strong pullup after next TMTouchByte 
      TMOneWireLevel(session_handle,LEVEL_SET,LEVEL_STRONG_PULLUP,PRIMED_BYTE);
         
      // send the last byte and do strong pullup
      if (TMTouchByte(session_handle,pbuf[cnt-1]) != pbuf[cnt-1])
         return FALSE;
         
      // sleep for a 10 ms
      Sleep(10);

      // disable the strong pull-up 
      TMOneWireLevel(session_handle,LEVEL_SET,LEVEL_NORMAL,PRIMED_NONE);
   
      // do one more byte to see if it is complete
      ch = (uchar)TMTouchByte(session_handle,0xFF);

      // return true if it is toggling 
      return ((ch == 0xAA) || (ch == 0x55));
   }
   else
      return FALSE;
}

//--------------------------------------------------------------------------
// Read memory (no checking)
//    
// Returns: TRUE (1) success
//          FALSE (0) failed
//
int ReadMemory(int num_pgs, uchar *buf)
{
   int i,cnt=0,rt=0;
   uchar pbuf[40];
   short printmsg=1,len;
   long session_handle;                         

   // loop to get a session handle
   for (;;)
   {
      // attempt to get a session 
      session_handle = TMExtendedStartSession(PortNum,PortType,NULL);
      if (session_handle > 0)  
      {
         // construct a packet to send  
         if (TMAccess(session_handle,&state_buffer) == 1)
         {
            pbuf[cnt++] = 0xF0; // write scratch command 
            pbuf[cnt++] = 0x00; // address 1 
            pbuf[cnt++] = 0x00; // address 2 

            // perform the block to start the read 
            if (TMBlockStream(session_handle,pbuf,(short)(cnt)) != 
                              (short)(cnt))
               break;

            // fill in the 0xFF's in the buf to use as a read buffer
            len = (num_pgs * 32);
            for (i = 0; i < len; i++)
              buf[i] = 0xFF;
      
            // perform the block to start the read 
            if (TMBlockStream(session_handle,buf,len) == len)
               rt = 1;

            break;
         }
      }
      else if (session_handle == 0)
      {                            
         // check if need to print the waiting message 
         if (printmsg)
         {  
            printmsg = FALSE;
            printf("\nWaiting to get access to the 1-Wire Net\n\n");
         }
      }
      else if (session_handle < 0)   
      {
         MessageBeep(MB_ICONEXCLAMATION);
         printf("No Hardware Driver for this type found!");
         break;
      }
   }

   // end the session if one is open 
   TMEndSession(session_handle);

   return rt;
}

//--------------------------------------------------------------------------
// Calculate a new CRC16 from the input data integer.  Return the current
// CRC16 and also update the global variable CRC16.
//
// 'data' - data to perform a CRC16 on
//
// Returns: the current CRC16
//
static short oddparity[16] = { 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0 };

ushort crc16(int data)
{
   data = (data ^ (CRC16 & 0xff)) & 0xff;
   CRC16 >>= 8;

   if (oddparity[data & 0xf] ^ oddparity[data >> 4])
      CRC16 ^= 0xc001;

   data <<= 6;
   CRC16   ^= data;
   data <<= 1;
   CRC16   ^= data;

   return CRC16;
}


